<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Admin
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Admin
{
    /**
     * Admin constructor.
     */
    public function __construct()
    {
        add_action('admin_menu', [$this, 'add_admin_menu'], 99);
        add_action('admin_init', [$this, 'settings_init']);
    }

    function add_admin_menu()
    {
        add_submenu_page('woocommerce', 'MultiParcels', 'MultiParcels', 'manage_options',
            'multiparcels-shipping-for-woocommerce', [$this, 'options_page']);
    }

    function validate($options)
    {
        /**
         * Force permission update if the API key has changed
         */
        if ($options['api_key'] != MultiParcels()->options->get('api_key')) {
            MultiParcels()->permissions->set(null);
        }

        return $options;
    }

    function settings_init()
    {
        register_setting('multiparcels-shipping-for-woocommerce', 'multiparcels_settings', [$this, 'validate']);

        add_settings_section(
            'multiparcels_multiparcels-shipping-for-woocommerce_section',
            null,
            [$this, 'multiparcels_settings_section_callback'],
            'multiparcels-shipping-for-woocommerce'
        );

        add_settings_field(
            'multiparcels_api_key',
            __('API key', 'multiparcels-shipping-for-woocommerce'),
            [$this, 'api_key_field_render'],
            'multiparcels-shipping-for-woocommerce',
            'multiparcels_multiparcels-shipping-for-woocommerce_section'
        );

        add_settings_field(
            'multiparcels_permissions',
            __('Permissions', 'multiparcels-shipping-for-woocommerce'),
            [$this, 'permissions_field_render'],
            'multiparcels-shipping-for-woocommerce',
            'multiparcels_multiparcels-shipping-for-woocommerce_section'
        );

        add_settings_field(
            'multiparcels_carriers',
            __('Carriers', 'multiparcels-shipping-for-woocommerce'),
            [$this, 'carrier_field_render'],
            'multiparcels-shipping-for-woocommerce',
            'multiparcels_multiparcels-shipping-for-woocommerce_section'
        );

        add_settings_field(
            'multiparcels_update',
            __('Data update', 'multiparcels-shipping-for-woocommerce'),
            [$this, 'update_field_render'],
            'multiparcels-shipping-for-woocommerce',
            'multiparcels_multiparcels-shipping-for-woocommerce_section'
        );

        add_settings_field(
            'multiparcels_terminals',
            __('Pickup locations', 'multiparcels-shipping-for-woocommerce'),
            [$this, 'terminals_list_render'],
            'multiparcels-shipping-for-woocommerce',
            'multiparcels_multiparcels-shipping-for-woocommerce_section'
        );
    }

    public function permissions_field_render()
    {
        $permissions      = MultiParcels()->permissions->get();
        $permission_title = sprintf("<strong>%s</strong>",
            __("None. Wrong API key", 'multiparcels-shipping-for-woocommerce'));

        if (MultiParcels()->permissions->isLimitedStrictly()) {
            $permission_title = __("Limited (terminals/pickup points only)", 'multiparcels-shipping-for-woocommerce');
        }

        if (MultiParcels()->permissions->isFull()) {
            $permission_title = __("Full", 'multiparcels-shipping-for-woocommerce');
        }

        echo $permission_title;

        if ($permissions == null || MultiParcels()->permissions->is_none()) {
            ?>
            <br>
            <br>
            <a href="#TB_inline?width=600&height=550&inlineId=my-content-id" class="thickbox button button-primary">
                <?php _e('Get a free API key', 'multiparcels-shipping-for-woocommerce'); ?>
            </a>
            <?php
        }
    }

    public function terminals_list_render()
    {
        $locations = MultiParcels()->locations->all();

        if (isset($_REQUEST['show_locations_list']) && $_REQUEST['show_locations_list'] == 1) {
            ?>
            <a href="?page=multiparcels-shipping-for-woocommerce" class="button">
                <?php _e('Hide', 'multiparcels-shipping-for-woocommerce') ?>
            </a>

            <div class="locations-list">
                <div class="locations-list-item header">
                    <div><?php _e('Carrier', 'multiparcels-shipping-for-woocommerce') ?></div>
                    <div><?php _e('Identification', 'multiparcels-shipping-for-woocommerce') ?></div>
                    <div><?php _e('Type', 'multiparcels-shipping-for-woocommerce') ?></div>
                    <div><?php _e('Name', 'multiparcels-shipping-for-woocommerce') ?></div>
                    <div><?php _e('Address', 'multiparcels-shipping-for-woocommerce') ?></div>
                    <div><?php _e('Postal code', 'multiparcels-shipping-for-woocommerce') ?></div>
                    <div><?php _e('City', 'multiparcels-shipping-for-woocommerce') ?></div>
                    <div><?php _e('Country', 'multiparcels-shipping-for-woocommerce') ?></div>
                </div>

                <?php
                foreach ($locations as $location) {
                    ?>
                    <div class="locations-list-item">
                        <div><?php echo MultiParcels()->carriers->name($location['carrier_code']); ?></div>
                        <div><?php echo $location['identifier']; ?></div>
                        <div><?php echo MultiParcels()->locations->type_name($location['type']); ?></div>
                        <div><?php echo $location['name']; ?></div>
                        <div><?php echo $location['address']; ?></div>
                        <div><?php echo $location['postal_code']; ?></div>
                        <div><?php echo $location['city']; ?></div>
                        <div><?php echo $location['country_code']; ?></div>
                    </div>
                <?php } ?>
            </div>

            <style>
                .locations-list {
                    display: flex;
                    width: 100%;
                    flex-direction: column;
                }

                .locations-list > div {
                    flex: 1;
                }

                .locations-list-item {
                    width: 100%;
                    display: flex;
                }

                .locations-list-item > div {
                    flex: 1;
                    padding: 3px;
                    border: 1px solid #ccc;
                }

                .locations-list-item.header {
                    font-weight: bold;
                }
            </style>
            <?php
        } else {
            ?>
            <a href="?page=multiparcels-shipping-for-woocommerce&amp;show_locations_list=1" class="button">
                <?php _e('Show', 'multiparcels-shipping-for-woocommerce') ?>
            </a>
            <?php
        }
    }

    public function carrier_field_render()
    {
        $carriers = MultiParcels()->carriers->all();

        if (MultiParcels()->permissions->isLimited()) {
            if (is_array($carriers)) {
                foreach ($carriers as $carrier => $settings) {
                    $enabled = MultiParcels()->options->getBool($carrier);

                    ?>
                    <div style="margin-bottom: 15px;display: flex;width: 40%;border-bottom: 1px solid #DDDDDD;padding-bottom: 15px;">
                        <div style="flex: 1;justify-content:  center;align-items: center;display:  flex">
                            <img src="https://platform.multiparcels.com/images/couriers/<?php echo esc_attr($carrier); ?>.png"
                                 style="width: 150px;">
                        </div>
                        <?php if ($enabled) { ?>
                            <div style="flex: 1;justify-content:  center;align-items: center;display:  flex; flex-direction: column">
                                <div style="margin: 5px 0;">
                                    <a href="<?php echo admin_url('admin.php?page=wc-settings&tab=shipping&section=multiparcels_' . $carrier . '_courier'); ?>"
                                       class="button">
                                        <?php _e('Courier Settings', 'multiparcels-shipping-for-woocommerce') ?>
                                    </a>
                                </div>
                                <?php if ($settings['has_terminals'] || $settings['has_pickup_points']) { ?>
                                    <div style="margin: 5px 0;">
                                        <a href="<?php echo admin_url('admin.php?page=wc-settings&tab=shipping&section=multiparcels_' . $carrier . '_pickup_points'); ?>"
                                           class="button">
                                            <?php _e('Pickup Settings', 'multiparcels-shipping-for-woocommerce') ?>
                                        </a>
                                    </div>
                                <?php } ?>
                            </div>
                            <div style="padding-left:20px;flex: 1;justify-content:  center;align-items: center;display:  flex;">
                                <a href="<?php echo admin_url('admin-post.php?action=multiparcels_carrier_change&change=disable&carrier=' . $carrier); ?>"
                                   class="button">
                                    <?php _e('Disable', 'multiparcels-shipping-for-woocommerce') ?>
                                </a>
                            </div>
                        <?php } else { ?>
                            <div style="flex: 1;justify-content:  center;align-items: center;display:  flex;">
                                <a href="<?php echo admin_url('admin-post.php?action=multiparcels_carrier_change&change=enable&carrier=' . $carrier); ?>"
                                   class="button button-primary">
                                    <?php _e('Enable', 'multiparcels-shipping-for-woocommerce') ?>
                                </a>
                            </div>
                            <div style="flex: 1;justify-content:  center;align-items: center;display:  flex;"></div>
                        <?php } ?>
                    </div>
                    <?php
                }
            } else {
                echo sprintf("<strong style='%s'>%s</strong>",
                    'color: red',
                    __('Please update the data to receive the carriers list', 'multiparcels-shipping-for-woocommerce'));
            }
        }
    }

    function api_key_field_render()
    {
        $value = MultiParcels()->options->get('api_key');
        ?>
        <input type='text' name='multiparcels_settings[api_key]' value='<?php echo $value; ?>'>
        <?php

        $permissions = MultiParcels()->permissions->get();

        if ($value == null || $permissions == null || MultiParcels()->permissions->is_none()) {
            $auto_show = true;

            /**
             * If the API key is provided but wrong - have free API key form ready
             */
            if ($permissions == null || MultiParcels()->permissions->is_none()) {
                $auto_show = false;
            }

            ?>
            <?php add_thickbox(); ?>
            <div id="my-content-id" style="display: none">
                <h3>
                    <?php _e('Get your free API key for terminals/pickup points!',
                        'multiparcels-shipping-for-woocommerce') ?>
                </h3>
                <h4>
                    <?php _e('Features that come with the free version:', 'multiparcels-shipping-for-woocommerce') ?>
                </h4>
                <ol>
                    <li>
                        <?php _e('Easy to configure', 'multiparcels-shipping-for-woocommerce') ?>
                    </li>
                    <li>
                        <?php _e('Costs 0&euro; per month', 'multiparcels-shipping-for-woocommerce') ?>
                    </li>
                    <li>
                        <?php _e('No credit card required', 'multiparcels-shipping-for-woocommerce') ?>
                    </li>

                    <li>
                        <?php _e('Automatic daily updated pickup locations', 'multiparcels-shipping-for-woocommerce') ?>
                    </li>
                    <li>
                        <?php _e('Configure every carrier the way you want', 'multiparcels-shipping-for-woocommerce') ?>
                    </li>
                    <li>
                        <?php _e('Free shipping option', 'multiparcels-shipping-for-woocommerce') ?>
                    </li>
                    <li>
                        <?php _e("Shows only possible locations for customer's city",
                            'multiparcels-shipping-for-woocommerce') ?>
                    </li>
                    <li>
                        <?php _e('New carriers without updating the plugin', 'multiparcels-shipping-for-woocommerce') ?>
                    </li>
                    <li>
                        <?php _e('Do you even need more features?', 'multiparcels-shipping-for-woocommerce') ?> <span
                                class="dashicons dashicons-smiley"></span>
                    </li>
                </ol>
                <p>
                    <?php _e('E-mail', 'multiparcels-shipping-for-woocommerce') ?>:

                    <input id="multiparcels-setup-email" type="text" name="email"
                           value="<?php echo esc_attr(wp_get_current_user()->get('user_email')); ?>"><br><br>

                    <button id="multiparcels-setup-submit" type="button" class="button button-primary">
                        <?php _e('Get the free API key', 'multiparcels-shipping-for-woocommerce') ?>
                    </button>
                </p>
            </div>

            <script type="text/javascript">
                jQuery(document).ready(function ($) {
                    $('#multiparcels-setup-submit').on('click', function () {
                        event.preventDefault();
                        var newForm = $('<form>', {
                            'action': '<?php echo admin_url('admin-post.php?action=multiparcels_request_api_key');?>',
                            'method': 'post'
                        }).append($('<input>', {
                            'name': 'email',
                            'value': $("#multiparcels-setup-email").val(),
                            'type': 'hidden'
                        }));
                        $(document.body).append(newForm);
                        newForm.submit();
                    });
                })
            </script>

            <?php if ($auto_show) { ?>
                <script type="text/javascript">
                    window.onload = function () {
                        tb_show("<?php echo __("MultiParcels Shipping For WooCommerce",
                            'multiparcels-shipping-for-woocommerce');?>", "#TB_inline?inlineId=my-content-id", false);
                    };
                </script>
                <?php
            }
        }
    }

    public function update_field_render()
    {
        $last_update = MultiParcels()->options->get('last_update');

        $next_cron_time = wp_next_scheduled('multiparcels_update_data_cron');
        if ($last_update === null) {
            $last_update = __('Never', 'multiparcels-shipping-for-woocommerce');
        }

        ?>
        <a href="<?php echo admin_url('admin-post.php?action=multiparcels_update_data'); ?>"
           class="button button-primary">
            <?php _e('Manual update', 'multiparcels-shipping-for-woocommerce') ?>
        </a>
        <br>
        <br>
        <?php _e('Last update', 'multiparcels-shipping-for-woocommerce') ?>:
        <strong><?php echo $last_update; ?></strong>

        <?php if ($next_cron_time != false) { ?>
        <br>
        <?php _e('Next update', 'multiparcels-shipping-for-woocommerce') ?>:
        <strong><?php echo date('Y-m-d H:i:s', $next_cron_time); ?></strong>
        <?php
    }

    }

    function multiparcels_settings_section_callback()
    {
    }

    function options_page()
    {
        if (isset($_REQUEST['settings-updated'])) {
            echo '<div class="updated"><p>' . __('Settings saved.',
                    'multiparcels-shipping-for-woocommerce') . '</p></div>';
        }

        ?>
        <form action='options.php' method='post'>
            <div>
                <img src="<?php echo MultiParcels()->public_plugin_url('images/logo.svg'); ?>" alt="MultiParcels"
                     style="width: 150px;margin-top: 15px;">
            </div>

            <?php
            settings_fields('multiparcels-shipping-for-woocommerce');
            do_settings_sections('multiparcels-shipping-for-woocommerce');

            $options = MultiParcels()->options->all();
            unset($options['api_key']);

            foreach ($options as $key => $option) {
                echo sprintf("<input type='hidden' name='%s' value='%s'/>", esc_attr($key), esc_attr($option));
            }
            submit_button();
            ?>

        </form>
        <?php

    }
}

return new MP_Admin();