<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Api_Client_Response
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Api_Client_Response
{
    /** @var bool Was the request successful? */
    protected $success = false;

    /** @var array */
    protected $data = [];

    /** @var array */
    protected $validation_errors = [];

    /** @var array|null */
    protected $full_response = [];

    /**
     * MP_Api_Client_Response constructor.
     *
     * @param string $output
     * @param int $httpCode
     */
    public function __construct($output, $httpCode)
    {
        $httpCodeGroup = floor($httpCode / 100);

        if ($decoded = json_decode($output, true)) {
            $this->full_response = $decoded;
        }

        if ($httpCode == 422) {
            $this->validation_errors = $this->full_response['errors'];
        } elseif ($httpCodeGroup == 2) {
            $this->success = true;

            if (array_key_exists('data', $this->full_response)) {
                $this->data = $this->full_response['data'];
            } else {
                $this->data = $this->full_response;
            }
        }
    }

    /**
     * @return bool
     */
    public function was_successful()
    {
        return $this->success;
    }

    /**
     * @return bool
     */
    public function has_validation_errors()
    {
        return (bool)count($this->validation_errors);
    }

    /**
     * @return array
     */
    public function get_validation_errors()
    {
        return $this->validation_errors;
    }

    /**
     * @return array
     */
    public function get_data()
    {
        return $this->data;
    }
}
