<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Carriers
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Carriers
{
    protected $carriers = [];

    /**
     * MP_Carriers constructor.
     */
    public function __construct()
    {
        $this->load();
    }

    /**
     * @param string $code
     *
     * @return mixed
     */
    public function get($code)
    {
        return $this->carriers[$code];
    }

    /**
     * @param string $code
     *
     * @return string
     */
    public function name($code)
    {
        if ( ! array_key_exists($code, $this->carriers)) {
            return $code;
        }

        return __($this->carriers[$code]['name'], 'multiparcels-shipping-for-woocommerce');
    }

    private function load()
    {
        $this->carriers = MultiParcels()->options->get('carriers', true);
    }

    /**
     * @return array
     */
    public function all()
    {
        return $this->carriers;
    }

    public function update()
    {
        $response = MultiParcels()->api_client->request('restricted_api/couriers', 'GET');

        if ($response->was_successful()) {
            $data     = $response->get_data();
            $carriers = $data['carriers'];

            MultiParcels()->options->set('carriers', $carriers, true);
            $this->carriers = $carriers;
        }
    }
}

return new MP_Carriers();
