<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Options
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Options
{
    /**
     * @var array
     */
    private $options = [];

    const OPTIONS_PREFIX = 'multiparcels_';
    const OPTIONS_KEY = 'multiparcels_settings';

    /**
     * Options constructor.
     */
    public function __construct()
    {
        $this->fillOptions();
    }

    /**
     * @param string $option Option to get
     *
     * @param string|null $other_settings
     *
     * @return mixed|null
     */
    public function get($option, $other_settings = null)
    {
        if ($other_settings == null && array_key_exists($option, $this->options)) {
            $value = $this->options[$option];

            if ($value == '') {
                $value = null;
            }

            return $value;
        }

        if ($other_settings != null) {
            $options = get_option(self::OPTIONS_PREFIX . $option);

            if ( ! $options) {
                return [];
            }

            return $options;
        }

        return null;
    }

    public function getBool($option)
    {
        return (bool)$this->get($option);
    }

    /**
     * Get all options
     *
     * @return array
     */
    public function all()
    {
        return $this->options;
    }

    /**
     * @param string $key
     * @param mixed $value
     * @param bool|null $other_settings
     */
    public function set($key, $value, $other_settings = false)
    {
        if ($other_settings == false) {
            $this->options[$key] = $value;

            update_option(self::OPTIONS_KEY, $this->options);
        } else {
            update_option(self::OPTIONS_PREFIX . $key, $value);
        }
    }

    /**
     * Remove the prefixes from keys
     */
    private function fillOptions()
    {
        $options = get_option(self::OPTIONS_KEY);
        if (is_array($options)) {
            $this->options = $options;
        }
    }

    /**
     * Helper for
     *
     * @return string|null
     */
    public function get_api_key()
    {
        return $this->get('api_key');
    }
}

return new MP_Options();
