<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Permissions
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Permissions
{
    const NONE = 'none';
    const LIMITED = 'limited';
    const FULL = 'full';

    /** @var string|null */
    protected $permission;

    /**
     * MP_Permissions constructor.
     */
    public function __construct()
    {
        $this->permission = MultiParcels()->options->get('permissions', true);

        if ($this->permission == null) {
            $this->update();
        }
    }

    public function get()
    {
        return $this->permission;
    }

    /**
     * @return bool
     */
    public function isFull()
    {
        return $this->permission == self::FULL;
    }

    /**
     * If the feature is only for limited - allow it for full too
     *
     * @return bool
     */
    public function isLimited()
    {
        return in_array($this->permission, [self::LIMITED, self::FULL]);
    }

    /**
     * Check to only make limited permissions pass
     *
     * @return bool
     */
    public function isLimitedStrictly()
    {
        return $this->permission == self::LIMITED;
    }

    public function update()
    {
        $response = MultiParcels()->api_client->request('restricted_api/permissions', 'POST', [
            'api_key' => MultiParcels()->options->get_api_key(),
        ]);

        if ($response->was_successful()) {
            $data        = $response->get_data();
            $permissions = $data['permission'];

            MultiParcels()->options->set('permissions', $permissions, true);
            $this->permission = $permissions;
        }
    }

    public function set($permissions)
    {
        MultiParcels()->options->set('permissions', $permissions, true);
    }

    public function is_none()
    {
        return $this->permission == self::NONE;
    }
}

return new MP_Permissions();