<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Woocommerce_Order_Shipping
 *
 * Enabling this does not actually make these features work.
 * All of the features here are actually doing all the work on the API, sorry :(
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Woocommerce_Order_Shipping
{
    /**
     * Package sizes
     */
    const PACKAGE_SIZE_SMALL = 'small';
    const PACKAGE_SIZE_MEDIUM = 'medium';
    const PACKAGE_SIZE_LARGE = 'large';
    const PACKAGE_SIZE_EXTRA_LARGE = 'extra-large';

    const PACKAGE_SIZES = [
        self::PACKAGE_SIZE_SMALL,
        self::PACKAGE_SIZE_MEDIUM,
        self::PACKAGE_SIZE_LARGE,
        self::PACKAGE_SIZE_EXTRA_LARGE,
    ];

    /**
     * Shipping methods
     */
    const SHIPPING_HANDS_TO_HANDS = 'hands_to_hands';
    const SHIPPING_HANDS_TO_TERMINAL = 'hands_to_terminal';
    const SHIPPING_TERMINAL_TO_TERMINAL = 'terminal_to_terminal';
    const SHIPPING_TERMINAL_TO_HANDS = 'terminal_to_hands';

    const SHIPPING_METHODS = [
        self::SHIPPING_HANDS_TO_HANDS,
        self::SHIPPING_HANDS_TO_TERMINAL,
        self::SHIPPING_TERMINAL_TO_TERMINAL,
        self::SHIPPING_TERMINAL_TO_HANDS,
    ];

    /**
     * Other constants
     */
    const CONFIRMED_KEY = 'multiparcels_confirmed';
    const EXTERNAL_ID_KEY = 'multiparcels_external_id';
    const TRACKING_CODE_KEY = 'multiparcels_tracking_code';
    const LABEL_LINK_KEY = 'multiparcels_label_link';

    /** @var WC_Order */
    private $order;

    /** @var array */
    private $order_meta;

    /** @var array|null */
    private $location;

    /** @var string */
    private $courier;

    /** @var int|null */
    private $order_id;

    /**
     * @param $order_id
     */
    public function reset($order_id)
    {
        $this->order      = wc_get_order($order_id);
        $this->order_meta = get_post_meta($order_id);
        $this->set_confirmed(false);
        $this->set_external_id(null);
        $this->set_tracking_code(null);
        $this->set_label_link(null);

        wp_redirect(admin_url('post.php?post=' . $this->order_id . '&action=edit'));
        exit;
    }


    /**
     * @param int $order_id
     */
    public function show($order_id)
    {
        $this->load_order($order_id);

        $data                         = [];
        $data['identifier']           = sprintf("woocommerce-%d-%d", $order_id, mt_rand(1000, 9999));

        if (version_compare(WC_VERSION, '3.0.0', '>=')) {
            $customer_note = $this->order->get_customer_note();
        } else {
            $customer_note = $this->order->customer_message;
        }

        $data['delivery']['comments'] = sanitize_text_field($customer_note);
        $data['receiver']             = $this->get_receiver();

        $this->display_status();
        echo str_repeat("<br/>", 2);

        if ( ! $this->is_confirmed()) {
            $this->display_size();
            echo str_repeat("<br/>", 2);

            $this->display_shipping();
            echo str_repeat("<br/>", 2);

            ?>
            <input type="hidden" id="multiparcels-sumbit-field" name="multiparcels_shipping[submit]" value="0">
            <input id="multiparcels-submit" type="button" class="button button-primary" name="save"
                   value="<?php _e('Confirm', 'multiparcels-shipping-for-woocommerce'); ?>">
            <script>
                jQuery(document).ready(function ($) {
                    $('#multiparcels-submit').on('click', function () {
                        $("#multiparcels-sumbit-field").val(1);
                        $("#post").submit();
                    });
                });
            </script>
            <?php
        } else {
            ?>
            <input type="hidden" id="multiparcels-sumbit-field" name="multiparcels_shipping[submit]" value="0">
            <input id="multiparcels-reset" type="button" class="button button-primary" name="save"
                   value="Reset">
            <script>
                jQuery(document).ready(function ($) {
                    $('#multiparcels-reset').on('click', function () {
                        $("#multiparcels-sumbit-field").val(2);
                        $("#post").submit();
                    });
                });
            </script>
            <?php
        }
    }

    /**
     * @return bool
     */
    public function is_confirmed()
    {
        if (array_key_exists(self::CONFIRMED_KEY, $this->order_meta)) {
            $confirmed = $this->order_meta[self::CONFIRMED_KEY][0];
            if ($confirmed) {
                return true;
            }
        }

        return false;
    }

    /**
     * @return string|null
     */
    public function get_external_id()
    {
        if (array_key_exists(self::EXTERNAL_ID_KEY, $this->order_meta)) {
            $id = $this->order_meta[self::EXTERNAL_ID_KEY][0];
            if ($id) {
                return $id;
            }
        }

        return null;
    }

    /**
     * @param bool $bool
     */
    public function set_confirmed($bool)
    {
        update_post_meta($this->order_id, self::CONFIRMED_KEY, (int)$bool);
    }

    /**
     * @return bool
     */
    public function get_label_link()
    {
        if (array_key_exists(self::LABEL_LINK_KEY, $this->order_meta)) {
            $link = $this->order_meta[self::LABEL_LINK_KEY][0];
            if ($link) {
                return $link;
            }
        }

        return false;
    }

    /**
     * @param string $link
     */
    public function set_label_link($link)
    {
        update_post_meta($this->order_id, self::LABEL_LINK_KEY, $link);
    }

    /**
     * @param string $external_id
     */
    public function set_external_id($external_id)
    {
        update_post_meta($this->order_id, self::EXTERNAL_ID_KEY, $external_id);
    }

    public function get_tracking_code()
    {
        if (array_key_exists(self::TRACKING_CODE_KEY, $this->order_meta)) {
            $tracking_code = $this->order_meta[self::TRACKING_CODE_KEY][0];
            if ($tracking_code) {
                return $tracking_code;
            }
        }

        return null;
    }

    /**
     * @param string $tracking_code
     */
    public function set_tracking_code($tracking_code)
    {
        update_post_meta($this->order_id, self::TRACKING_CODE_KEY, $tracking_code);
    }

    /**
     * @param $bool
     *
     * @return string
     */
    private function bool_text($bool)
    {
        if ($bool) {
            return __('Yes', 'multiparcels-shipping-for-woocommerce');
        }

        return __('No', 'multiparcels-shipping-for-woocommerce');
    }

    private function display_status()
    {
        if ($this->is_confirmed()) {
            echo sprintf("<strong>%s:</strong> %s (<a href='%s' target='_blank'>%s</a>)<br/>",
                __('Confirmed', 'multiparcels-shipping-for-woocommerce'),
                esc_attr($this->bool_text($this->is_confirmed())),
                esc_attr('https://platform.multiparcels.com/shipments/' . $this->get_external_id()),
                __('View on Platform', 'multiparcels-shipping-for-woocommerce')
            );

            echo sprintf("<strong>%s:</strong> %s<br/>",
                __('Tracking code', 'multiparcels-shipping-for-woocommerce'),
                esc_html($this->get_tracking_code()));

        } else {
            echo sprintf("<strong>%s:</strong> %s<br/>",
                __('Confirmed', 'multiparcels-shipping-for-woocommerce'),
                esc_html($this->bool_text($this->is_confirmed())));
        }
        if ($label_link = $this->get_label_link()) {
            $label_link = wp_upload_dir()['baseurl'] . '/' . $label_link;

            echo sprintf("<strong>%s:</strong> <a href='%s' target='_blank'>%s</a><br/>",
                __('Label', 'multiparcels-shipping-for-woocommerce'),
                esc_attr($label_link),
                _x('View', 'label', 'multiparcels-shipping-for-woocommerce')
            );
        } else {
            echo sprintf("<strong>%s:</strong> %s<br/>",
                __('Label', 'multiparcels-shipping-for-woocommerce'),
                __('No', 'multiparcels-shipping-for-woocommerce'));
        }
    }

    public function show_after_shipping_address_info($order_id)
    {
        $this->load_order($order_id);

        if ( ! $this->is_confirmed()) {
            echo sprintf('<a class="button button-primary" href="%s">%s</a>',
                esc_attr('#multiparcels-shipping-box'),
                __('Ship order', 'multiparcels-shipping-for-woocommerce'));
        }
    }

    /**
     * Not used at the moment
     *
     * @param array $receiver
     */
    private function display_receiver($receiver)
    {
        echo "<strong>" . __("Receiver", 'multiparcels-shipping-for-woocommerce') . "</strong> <br/>";
        echo sprintf("%s <br/>", $receiver['name']);
        echo sprintf("%s, %s, %s, %s <br/>", $receiver['street'], $receiver['city'], $receiver['country_code'],
            $receiver['postal_code']);
        echo sprintf("%s %s<br/>", $receiver['email'], $receiver['phone_number']);
    }

    private function display_size()
    {
        $sizes = [
            self::PACKAGE_SIZE_SMALL,
            self::PACKAGE_SIZE_MEDIUM,
            self::PACKAGE_SIZE_LARGE,
            self::PACKAGE_SIZE_EXTRA_LARGE,
        ];

        echo sprintf("<strong>%s</strong><br><br>", __('Package size', 'multiparcels-shipping-for-woocommerce'));
        echo "<div style='display: flex;'>";
        foreach ($sizes as $size) {
            ?>
            <div style="margin-right: 30px;">
                <input type="radio" name="multiparcels_shipping[package_sizes]"
                       value="<?php echo $size; ?>" <?php echo checked($size, 'small'); ?>><br>
                <img class="check_closest"
                     src='<?php echo MultiParcels()->public_plugin_url('images/boxes/' . $size . '.png'); ?>'
                     style="width: 150px;cursor: pointer"/>
            </div>
            <?php
        }
        echo "</div>";
        ?>
        <script>
            jQuery(document).ready(function ($) {
                $('.check_closest').on('click', function () {
                    $(this).parent().find('input').prop('checked', true);

                    return false;
                });
            });
        </script>
        <?php
    }

    private function display_shipping()
    {
        $types = [
            self::SHIPPING_HANDS_TO_HANDS,
            self::SHIPPING_TERMINAL_TO_HANDS,
        ];

        $selected = $types[0];

        if ($this->location != null) {
            $selected = 'hands_to_terminal';

            $types = [
                self::SHIPPING_HANDS_TO_TERMINAL,
                self::SHIPPING_TERMINAL_TO_TERMINAL,
            ];
        }

        echo sprintf("<strong>%s</strong><br><br>", __('Shipping method', 'multiparcels-shipping-for-woocommerce'));
        echo "<div style='display: flex;'>";
        foreach ($types as $type) {
            $title = $this->shipping_type_title($type);
            ?>

            <div style="margin-right: 30px;">
                <input type="radio" name="multiparcels_shipping[shipping]"
                       value="<?php echo $type; ?>" <?php echo checked($type,
                    $selected); ?>> <?php echo $title; ?> <br>
                <img src='<?php echo MultiParcels()->public_plugin_url('images/shipping_methods/' . $type . '.png'); ?>'
                     style="width: 150px;cursor: pointer;margin-top: 10px;" class="check_closest"/>
            </div>
            <?php
        }
        echo "</div>";
    }

    /**
     * @param int $order_id
     * @param array $post
     */
    public function ship_order($order_id, $post)
    {
        $this->load_order($order_id);

        $methods = $this->parse_methods($post['shipping']);

        $data                              = [];
        $data['identifier']                = sprintf("woocommerce-%d-%d", $order_id, mt_rand(1000, 9999));
        $data['pickup']['type']            = $methods['pickup'];
        $data['pickup']['weight']          = 2;
        $data['pickup']['packages']        = 1;
        $data['pickup']['package_sizes']   = [
            $post['package_sizes'],
        ];
        $data['delivery']['type']          = $methods['delivery'];
        $data['delivery']['courier']       = $this->courier;
        $data['sender']['address_book_id'] = 'yj5ejv';
        $data['receiver']                  = $this->get_receiver();

        if ($this->location) {
            $data['receiver']['location_postal_code'] = $this->location['postal_code'];
        }

        $response = MultiParcels()->api_client->request('shipments', 'POST', $data);
        if ($response->was_successful()) {
            $shipment = $response->get_data();
            $this->set_confirmed(true);
            $this->set_external_id($shipment['id']);

            $confirm_link     = sprintf('shipments/%s/confirm', $shipment['id']);
            $confirm_response = MultiParcels()->api_client->request($confirm_link, 'POST');

            if ($confirm_response->was_successful()) {
                $confirm_data = $confirm_response->get_data();
                $this->set_tracking_code($confirm_data['tracking_codes'][0]);

                $labels_link     = sprintf('shipments/%s/labels', $shipment['id']);
                $labels_response = MultiParcels()->api_client->request($labels_link, 'GET');

                if ($labels_response->was_successful()) {
                    $data = $labels_response->get_data();

                    $file_name = sprintf("label_%d.pdf", $this->order_id);

                    $upload = wp_upload_bits($file_name, null, base64_decode($data['content']));
                    $this->set_label_link(explode('/wp-content/uploads/', $upload['url'])[1]);
                }
            }
        } else {
            echo 'failed';
            var_dump($response->get_validation_errors());
            exit;
        }

        wp_redirect(admin_url('post.php?post=' . $this->order_id . '&action=edit#multiparcels-shipping-box'));
        exit;
    }

    /**
     * @return array|null
     */
    private function get_location()
    {
        $shipping_methods = $this->order->get_shipping_methods();
        $shipping_methods = reset($shipping_methods);

        // extract the courier
        $courier = str_replace(['multiparcels_', '_courier', '_pickup_points'], '', $shipping_methods['method_id']);

        return $this->location = MultiParcels()->locations->get_location_for_order($this->order, null, $courier);
    }

    /**
     * @return array
     */
    private function get_receiver()
    {
        $receiver                 = [];
        $receiver['name']         = sanitize_text_field(sprintf("%s %s", $this->order_meta['_shipping_first_name'][0],
            $this->order_meta['_shipping_last_name'][0]));
        $receiver['phone_number'] = sanitize_text_field($this->order_meta['_billing_phone'][0]);
        $receiver['street']       = sanitize_text_field($this->order_meta['_shipping_address_1'][0]);
        $receiver['city']         = sanitize_text_field($this->order_meta['_shipping_city'][0]);
        $receiver['postal_code']  = sanitize_text_field($this->order_meta['_shipping_postcode'][0]);
        $receiver['email']        = sanitize_text_field($this->order_meta['_billing_email'][0]);
        $receiver['country_code'] = sanitize_text_field($this->order_meta['_shipping_country'][0]);

        return $receiver;
    }

    /**
     * @param $shipping
     *
     * @return array
     */
    private function parse_methods($shipping)
    {
        $check = in_array($shipping, self::SHIPPING_METHODS);

        if ($check) {
            $explode = explode('_to_', $shipping);

            return [
                'pickup'   => $explode[0],
                'delivery' => $explode[1],
            ];
        }

        return [
            'pickup'   => '',
            'delivery' => '',
        ];
    }

    /**
     * @param $order_id
     */
    private function load_order($order_id)
    {
        $this->order      = wc_get_order($order_id);
        $this->order_meta = get_post_meta($order_id);

        if (version_compare(WC_VERSION, '3.0.0', '>=')) {
            $this->order_id = $this->order->get_id();
        } else {
            $this->order_id = $this->order->id;
        }

        $this->location   = $this->get_location();

        $shipping_methods = $this->order->get_shipping_methods();
        $shipping_methods = reset($shipping_methods);

        // extract the courier
        $this->courier = str_replace(['multiparcels_', '_courier', '_pickup_points'], '',
            $shipping_methods['method_id']);
    }

    private function shipping_type_title($type)
    {
        if ($type == self::SHIPPING_HANDS_TO_HANDS) {
            $type = __('Hands to hands', 'multiparcels-shipping-for-woocommerce');
        }

        if ($type == self::SHIPPING_HANDS_TO_TERMINAL) {
            $type = __('Hands to terminal', 'multiparcels-shipping-for-woocommerce');
        }

        if ($type == self::SHIPPING_TERMINAL_TO_TERMINAL) {
            $type = __('Terminal to terminal', 'multiparcels-shipping-for-woocommerce');
        }

        if ($type == self::SHIPPING_TERMINAL_TO_HANDS) {
            $type = __('Terminal to hands', 'multiparcels-shipping-for-woocommerce');
        }

        return $type;
    }

}

return new MP_Woocommerce_Order_Shipping();