<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Woocommerce
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Woocommerce
{
    /**
     * MP_Woocommerce constructor.
     */
    public function __construct()
    {
        add_filter('woocommerce_locate_template', [$this, 'locate_template'], 20, 3);
        add_filter('woocommerce_locate_core_template', [$this, 'locate_template'], 20, 3);
        add_filter('woocommerce_shipping_init', [$this, 'shipping_init']);
        add_filter('woocommerce_shipping_methods', [$this, 'register_shipping_methods']);

        /**
         * Add full access features only if they are enabled.
         * Enabling this does not actually make them work.
         * All of the features here are actually doing all the work on the API, sorry :(
         */
        if (MultiParcels()->permissions->isFull()) {
            add_filter('manage_edit-shop_order_columns', [$this, 'add_column_to_orders'], 20);
            add_action('manage_shop_order_posts_custom_column', [$this, 'add_content_to_column']);

            add_action('add_meta_boxes', [$this, 'add_meta_box_to_order'], 1);

            add_action('woocommerce_admin_order_data_after_shipping_address',
                [$this, 'woocommerce_admin_order_data_after_shipping_address'], 50);

            add_action('save_post', [$this, 'saving_order']);
            /**
             * @todo
             */
            // add_filter('bulk_actions-edit-shop_order', [$this, 'register_my_bulk_actions']);
        }
    }

    function saving_order($post_id)
    {
        if (isset($_POST['multiparcels_shipping']) && isset($_POST['multiparcels_shipping']['submit']) && $_POST['multiparcels_shipping']['submit'] == 1) {
            $data = array_map('sanitize_text_field', wp_unslash($_POST['multiparcels_shipping']));

            /** @var MP_Woocommerce_Order_Shipping $shipping */
            $shipping = new MP_Woocommerce_Order_Shipping();
            $shipping->ship_order($post_id, $data);
        }

        if (isset($_POST['multiparcels_shipping']) && isset($_POST['multiparcels_shipping']['submit']) && $_POST['multiparcels_shipping']['submit'] == 2) {

            /** @var MP_Woocommerce_Order_Shipping $shipping */
            $shipping = new MP_Woocommerce_Order_Shipping();
            $shipping->reset($post_id);
        }
    }

    /**
     * @param WC_Order $order
     */
    public function woocommerce_admin_order_data_after_shipping_address($order)
    {
        $shipping = new MP_Woocommerce_Order_Shipping();

        if (version_compare(WC_VERSION, '3.0.0', '>=')) {
            $order_id = $order->get_id();
        } else {
            $order_id = $order->id;
        }

        $shipping->show_after_shipping_address_info($order_id);
    }

    function add_meta_box_to_order()
    {
        $screens = ['shop_order'];

        foreach ($screens as $screen) {
            add_meta_box(
                'multiparcels-shipping-box',
                __('MultiParcels Platform', 'multiparcels-shipping-for-woocommerce'),  // Box title
                [$this, 'order_meta_box_content'],
                $screen
            );
        }
    }

    /**
     * @param WP_Post $post
     */
    function order_meta_box_content($post)
    {
        /** @var MP_Woocommerce_Order_Shipping $shipping */
        $shipping = new MP_Woocommerce_Order_Shipping();
        $shipping->show($post->ID);
    }

    /**
     * @param string $column
     */
    function add_content_to_column($column)
    {
        global $post;

        if ($column === 'multiparcels-shipping') {
            $order_id = $post->ID;

            $order = wc_get_order($post);

            $is_confirmed = (bool)get_post_meta($order_id, MP_Woocommerce_Order_Shipping::CONFIRMED_KEY, true);
            $label_link   = get_post_meta($order_id, MP_Woocommerce_Order_Shipping::LABEL_LINK_KEY, true);

            if ( ! $is_confirmed && $order->has_status(['processing'])) {
                echo sprintf('<a class="button button-primary" href="%s">%s</a>',
                    esc_attr(admin_url('post.php?action=edit&post=' . $order_id) . '#multiparcels-box'),
                    __('Ship order', 'multiparcels-shipping-for-woocommerce'));
            } else {
                if ($label_link) {
                    $label_link = esc_attr(wp_upload_dir()['baseurl'] . '/' . $label_link);

                    echo sprintf('<a class="button" href="%s" target="_blank">%s</a>',
                        $label_link,
                        __('Label', 'multiparcels-shipping-for-woocommerce'));
                }
            }
        }
    }

    /**
     * @param array $columns
     *
     * @return array
     */
    function add_column_to_orders($columns)
    {
        $new_columns = [];

        foreach ($columns as $column_name => $column_info) {
            $new_columns[$column_name] = $column_info;

            if ('order_total' === $column_name) {
                $new_columns['multiparcels-shipping'] = __('MultiParcels', 'multiparcels-shipping-for-woocommerce');
            }
        }

        return $new_columns;
    }

    /**
     * @param $methods
     *
     * @return mixed
     */
    static function register_shipping_methods($methods)
    {
        if (class_exists('WC_MP_Shipping_Method')) {
            foreach (MultiParcels()->options->get('carriers', true) as $code => $settings) {
                $enabled = MultiParcels()->options->getBool($code);
                if ($enabled) {

                    $key           = 'multiparcels_' . $code;
                    $methods[$key] = MultiParcels()->shipping_methods[$key];

                    if ($settings['has_courier_service']) {
                        $key           = 'multiparcels_' . $code . '_carrier';
                        $methods[$key] = MultiParcels()->shipping_methods[$key];
                    }
                }
            }
        }

        return $methods;
    }

    static public function shipping_init()
    {
        if (class_exists('WC_MP_Shipping_Method')) {
            foreach (MultiParcels()->options->get('carriers', true) as $code => $settings) {
                $enabled = MultiParcels()->options->getBool($code);

                if ($enabled) {
                    $key                                   = 'multiparcels_' . $code;
                    MultiParcels()->shipping_methods[$key] = new WC_MP_Shipping_Method($code, $settings);

                    if ($settings['has_courier_service']) {
                        $key                                   = 'multiparcels_' . $code . '_carrier';
                        MultiParcels()->shipping_methods[$key] = new WC_MP_Shipping_Method($code, $settings, true);
                    }
                }
            }
        }
    }

    function locate_template($template, $template_name, $template_path)
    {
        global $woocommerce;

        $_template = $template;

        if ( ! $template_path) {
            $template_path = $woocommerce->template_url;
        }

        $plugin_path = MultiParcels()->plugin_path() . '/woocommerce/';

        // Look within passed path within the theme - this is priority
        $template = locate_template(
            [
                $template_path . $template_name,
                $template_name,
            ]
        );

        // Modification: Get the template from this plugin, if it exists
        if ( ! $template && file_exists($plugin_path . $template_name)) {
            $template = $plugin_path . $template_name;
        }

        // Use default template
        if ( ! $template) {
            $template = $_template;
        }

        // Return what we found
        return $template;
    }

    /**
     * @todo https://make.wordpress.org/core/2016/10/04/custom-bulk-actions/
     */
    //    function register_my_bulk_actions($bulk_actions)
    //    {
    //        $bulk_actions['confirm'] = __('Confirm');
    //
    //        return $bulk_actions;
    //    }
}

return new MP_Woocommerce();