<?php
/**
 * Plugin Name: MultiParcels Shipping For WooCommerce
 * Description: Easiest, fastest and the cheapest way to integrate couriers with all deliveries methods to send parcels with just a few button clicks.
 * Version: 1.0.4
 * Author: MultiParcels
 * Author URI: https://multiparcels.com
 *
 * Text Domain: multiparcels-shipping-for-woocommerce
 * Domain Path: /languages/
 */

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MultiParcels
 *
 * @todo finish order shipped email
 * @todo order list select all that you want to send (mass shipping settings, must still pick sending methods)
 * @todo select default sender
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MultiParcels
{
    /**
     * The single instance of the class.
     *
     * @var MultiParcels
     * @since 0.1
     */
    protected static $_instance;

    /** @var MP_Options */
    public $options;

    /** @var MP_Api_Client */
    public $api_client;

    /** @var MP_Locations */
    public $locations;

    /** @var MP_Carriers */
    public $carriers;

    /** @var MP_Permissions */
    public $permissions;

    /**
     * @since 0.1
     * @return MultiParcels
     */
    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /** @var array */
    public $shipping_methods = [];

    /**
     * MultiParcels constructor.
     */
    function __construct()
    {
        add_action('plugins_loaded', [$this, 'plugins_loaded']);
        $this->load_plugin_textdomain();

        include_once 'includes/class-mp-install.php';
        register_activation_hook(__FILE__, ['MP_Install', 'install']);
        register_deactivation_hook(__FILE__, ['MP_Install', 'remove']);
    }

    public function plugins_loaded()
    {
        $this->includes();
        $this->init_hooks();
    }

    private function init_hooks()
    {
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), [__CLASS__, 'plugin_action_links']);
    }

    private function includes()
    {
        $this->options = include_once 'includes/class-mp-options.php';
        include_once 'includes/class-mp-api-client-response.php';
        $this->api_client  = include_once 'includes/class-mp-api-client.php';
        $this->locations   = include_once 'includes/class-mp-locations.php';
        $this->carriers    = include_once 'includes/class-mp-carriers.php';
        $this->permissions = include_once 'includes/class-mp-permissions.php';
        include_once 'includes/class-mp-admin.php';
        include_once 'includes/class-wc-mp-shipping-method.php';
        include_once 'includes/class-mp-actions.php';
        include_once 'includes/class-mp-woocommerce.php';
        include_once 'includes/class-mp-notices.php';
        include_once 'includes/class-mp-woocommerce-order-shipping.php';
    }

    /**
     * Show action links on the plugin screen.
     *
     * @param    mixed $links Plugin Action links
     *
     * @return    array
     */
    public static function plugin_action_links($links)
    {
        $action_links = [
            sprintf('<a href="%s" title="%s">%s</a>',
                admin_url('admin.php?page=multiparcels'),
                esc_attr(__('View MultiParcels Settings', 'multiparcels-shipping-for-woocommerce')),
                __('Settings', 'multiparcels-shipping-for-woocommerce')
            ),
        ];

        return array_merge($action_links, $links);
    }

    public function load_plugin_textdomain()
    {
        load_textdomain('multiparcels-shipping-for-woocommerce',
            WP_LANG_DIR . '/multiparcels-shipping-for-woocommerce/multiparcels-shipping-for-woocommerce-' . get_locale() . '.mo');
        load_plugin_textdomain('multiparcels-shipping-for-woocommerce', false,
            plugin_basename(dirname(__FILE__)) . '/languages');
    }

    /**
     * @return string
     */
    public function plugin_path()
    {
        return untrailingslashit(plugin_dir_path(__FILE__));
    }

    /**
     * @return string
     */
    function settings_url()
    {
        return esc_url(admin_url('admin.php?page=multiparcels-shipping-for-woocommerce'));
    }

    /**
     * Get the public plugin url.
     *
     * @param string|null $sub_path
     *
     * @return string
     */
    public function public_plugin_url($sub_path = null)
    {
        $path = untrailingslashit(plugins_url('/', __FILE__)) . '/public';

        if ($sub_path) {
            $path .= '/' . $sub_path;
        }

        return esc_url($path);
    }
}

/**
 * @return MultiParcels
 */
function MultiParcels()
{
    return MultiParcels::instance();
}

MultiParcels();
